@@ -390,7 +390,8 @@ impl super::Kubectl {
         or_insert_with: F,
     ) -> anyhow::Result<V>
     where
-        K: Ord + ser::Serialize + de::DeserializeOwned + Clone,
+        K: Ord + ToString + std::str::FromStr + Clone,
+        K::Err: std::error::Error + Send + Sync + 'static,
         V: ser::Serialize + de::DeserializeOwned + Clone,
         F: Fn(&BTreeMap<K, V>) -> anyhow::Result<V>,
     {
@@ -417,7 +418,8 @@ impl super::Kubectl {
         key: &K,
     ) -> anyhow::Result<Option<V>>
     where
-        K: Ord + ser::Serialize + de::DeserializeOwned,
+        K: Ord + ToString + std::str::FromStr,
+        K::Err: std::error::Error + Send + Sync + 'static,
         V: ser::Serialize + de::DeserializeOwned,
     {
         let mut value = None;
@@ -436,11 +438,12 @@ impl super::Kubectl {
         mut update_fn: impl FnMut(&mut BTreeMap<K, V>),
     ) -> anyhow::Result<corev1::ConfigMap>
     where
-        K: Ord + ser::Serialize + de::DeserializeOwned,
+        K: Ord + ToString + std::str::FromStr,
+        K::Err: std::error::Error + Send + Sync + 'static,
         V: ser::Serialize + de::DeserializeOwned,
     {
         // rv used to make sure that update has been applied successfully
-        let mut rv = Ok::<_, anyhow::Error>(());
+        let mut rv = Ok(());
         let map = self
             .update_config_map(name, |data| {
                 rv = try_update_json_configmap(data, &mut update_fn)
@@ -596,15 +599,16 @@ fn try_update_json_configmap<K, V>(
     mut update_fn: impl FnMut(&mut BTreeMap<K, V>),
 ) -> anyhow::Result<()>
 where
-    K: Ord + ser::Serialize + de::DeserializeOwned,
+    K: Ord + ToString + std::str::FromStr,
+    K::Err: std::error::Error + Send + Sync + 'static,
     V: ser::Serialize + de::DeserializeOwned,
 {
     use serde_json::{from_str, to_string};
     use std::any::type_name;
     let mut data: BTreeMap<K, V> = cm_data
         .iter()
-        .map(|(key, value)| Ok((from_str(key)?, from_str(value)?)))
-        .collect::<serde_json::Result<_>>()
+        .map(|(key, value)| Ok((key.parse()?, from_str(value)?)))
+        .collect::<anyhow::Result<_>>()
         .with_context(|| {
             format!(
                 "deserialize configmap {:?} data into BTreeMap<{}, {}>",
@@ -616,7 +620,7 @@ where
     update_fn(&mut data);
     *cm_data = data
         .iter()
-        .map(|(key, value)| Ok((to_string(key)?, to_string(value)?)))
+        .map(|(key, value)| Ok((key.to_string(), to_string(value)?)))
         .collect::<serde_json::Result<_>>()
         .with_context(|| format!("serialize configmap {:?} after update", cm_data))?;
 
